@@ -17,6 +17,7 @@
  * Rights Reserved.
  * 
  * Contributor(s):
+ *  John Gardiner Myers <jgmyers@speakeasy.net>
  * 
  * Alternatively, the contents of this file may be used under the
  * terms of the GNU General Public License Version 2 or later (the
@@ -32,7 +33,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: utf8.c,v $ $Revision: 1.3 $ $Date: 2003/11/27 05:08:20 $ $Name: NSS_3_9_RTM $";
+static const char CVS_ID[] = "@(#) $RCSfile: utf8.c,v $ $Revision: 1.7 $ $Date: 2004/01/20 19:57:17 $ $Name: NSS_3_9_1_RTM $";
 #endif /* DEBUG */
 
 #include "seccomon.h"
@@ -553,6 +554,56 @@ sec_port_ucs2_utf8_conversion_function
   }
 }
 
+PRBool
+sec_port_iso88591_utf8_conversion_function
+(
+  const unsigned char *inBuf,
+  unsigned int inBufLen,
+  unsigned char *outBuf,
+  unsigned int maxOutBufLen,
+  unsigned int *outBufLen
+)
+{
+  unsigned int i, len = 0;
+
+#ifndef TEST_UTF8
+  PORT_Assert((unsigned int *)NULL != outBufLen);
+#endif /* TEST_UTF8 */
+
+  for( i = 0; i < inBufLen; i++) {
+    if( (inBuf[i] & 0x80) == 0x00 ) len += 1;
+    else len += 2;
+  }
+
+  if( len > maxOutBufLen ) {
+    *outBufLen = len;
+    return PR_FALSE;
+  }
+
+  len = 0;
+
+  for( i = 0; i < inBufLen; i++) {
+    if( (inBuf[i] & 0x80) == 0x00 ) {
+      /* 00-7F -> 0xxxxxxx */
+      /* 0abcdefg -> 0abcdefg */
+
+      outBuf[len] = inBuf[i];
+      len += 1;
+    } else {
+      /* 80-FF <- 110xxxxx 10xxxxxx */
+      /* 00000000 abcdefgh -> 110000ab 10cdefgh */
+
+      outBuf[len+0] = 0xC0 | ((inBuf[i] & 0xC0) >> 6);
+      outBuf[len+1] = 0x80 | ((inBuf[i] & 0x3F) >> 0);
+
+      len += 2;
+    }
+  }
+
+  *outBufLen = len;
+  return PR_TRUE;
+}
+
 #ifdef TEST_UTF8
 
 #include <stdio.h>
@@ -1742,6 +1793,51 @@ test_utf16_chars
 }
 #endif /* UTF16 */
 
+static PRBool
+test_iso88591_chars
+(
+  void
+)
+{
+  PRBool rv = PR_TRUE;
+  int i;
+
+  for( i = 0; i < sizeof(ucs2)/sizeof(ucs2[0]); i++ ) {
+    struct ucs2 *e = &ucs2[i];
+    PRBool result;
+    unsigned char iso88591;
+    unsigned char utf8[3];
+    unsigned int len = 0;
+
+    if (e->c > 0xFF) continue;
+
+    (void)memset(utf8, 0, sizeof(utf8));
+    iso88591 = e->c;
+    
+    result = sec_port_iso88591_utf8_conversion_function(&iso88591,
+      1, utf8, sizeof(utf8), &len);
+
+    if( !result ) {
+      fprintf(stdout, "Failed to convert ISO-8859-1 0x%02.2x to UTF-8\n", iso88591);
+      rv = PR_FALSE;
+      continue;
+    }
+
+    if( (len >= sizeof(utf8)) ||
+        (strlen(e->utf8) != len) ||
+        (utf8[len] = '\0', 0 != strcmp(e->utf8, utf8)) ) {
+      fprintf(stdout, "Wrong conversion of ISO-8859-1 0x%02.2x to UTF-8: ", iso88591);
+      dump_utf8("expected", e->utf8, ", ");
+      dump_utf8("received", utf8, "\n");
+      rv = PR_FALSE;
+      continue;
+    }
+
+  }
+
+  return rv;
+}
+
 static PRBool
 test_zeroes
 (
@@ -2057,6 +2153,7 @@ main
 #ifdef UTF16
       test_utf16_chars() &&
 #endif /* UTF16 */
+      test_iso88591_chars() &&
       test_zeroes() &&
       test_multichars() &&
       PR_TRUE ) {
